<?php
/**
 * ページネーション
 *
 * CakePHPのPaginatorHelperを参考にしたページングクラスです
 * ページ番号・最大ページ数・データ件数・表示件数を元にナビゲーションを作成します
 *           
 * LICENSE : MIT License
 *
 * @category  Step8
 * @package   OOP-BBS
 * @author    Tenderfeel <tenderfeel@gmail.com>
 * @copyright Copyright (c) 2010 Tenderfeel.(http://webtecnote.com)
 *            Copyright 2005-2010, Cake Software Foundation, Inc. (http://cakefoundation.org)
 * @license   MIT License (http://www.opensource.org/licenses/mit-license.php)
 * @since     File available since Release OOP-BBS v 1.8.0 (step8)
 *
 */
 
class Pagination {
    
    /**
     *  - `$params['page']` ページ番号
     *  - `$params['pageCount']` 最大ページ数
     *  - `$params['dataCount']` データ件数
     *  - `$params['limit']` 表示件数
     *  - `$params['url']` URL
     *  - `$params['paramKey']` パラメーターキー
     */
    public $params  = array();
    /**
     * - `$options['format']` counterメソッド用のフォーマット。タグを使って値を表示できます。
     *                        %page%, %pages%, %current%, %count%, %start%, %end% .
     * - `$options['separator']` ページナンバーと総ページ数のセパレーター
     */
    public $options = array();
    
    const PARAM_KEY = '?page=';
    
    /**
     * コンストラクタ
     * @param $params array　ページ番号・最大ページ数・データ件数・表示件数
     */
    function __construct($params)
    {
        $this->params = $params;
        
        if(empty($this->params['paramKey'])){
            $this->params['paramKey'] = self::PARAM_KEY;
        }
        
        if(empty($this->params['url'])){
            $this->params['url'] = '';
        }
        
        $this->params['next'] = $this->params['page'] + 1;
        $this->params['prev'] = $this->params['page'] - 1;
    }
    
    public function options($options)
    {
        $this->options = $options;
    }
    
    /**
     * 前のページへ　のリンク
     * 
     * @param string $title リンクのタイトル。デフォルトは'&laquo; 前のページへ'
     * @param array $options　オプション
     */
    public function prev($title = '&laquo; 前へ', $options = array())
    {
        $this->_pagingLink('Prev', $title, $options);
    }
    
    /**
     * 次のページへ　のリンク
     * 
     * @param string $title リンクのタイトル。デフォルトは'&laquo; 前のページへ'
     * @param array $options　オプション
     */
    public function next($title = '次へ &raquo;', $options = array())
    {
        
        $this->_pagingLink('Next', $title, $options);
    }
    
    /**
     * 前のページへ/次のページへ　リンク作成
     * 
     * @param string $which prev/next
     * @param string $title リンクのタイトル
     * @param array $options　オプション
     */
    protected function _pagingLink($which, $title = null, $options = array())
    {
        $check = 'has' . $which;
        $_defaults = array('tag' => 'span', 'class' => strtolower($which));
        $options = array_merge($_defaults, (array)$options);
        
        $tag = '<' . $options['tag'] . ' class="' . $options['class'] . '">';
        $tag .= '<a href="' . $this->params['url']. $this->params['paramKey'] . $this->params[$options['class']]. '">' . $title . '</a>';
        $tag .= '</' . $options['tag'] . '>';
        
        if($this->{$check}())  print $tag;
    }
    
    /**
     * 前のページの有無
     * 
     * @return boolean 前のページがあればtrue、なければfalse
     */
    public function hasNext()
    {
        return $this->_hasPage("next");
    }
    
    /**
     * 次のページの有無
     * 
     * @return boolean 次のページがあればtrue、なければfalse
     */
    public function hasPrev()
    {
        return $this->_hasPage("prev");
    }
    
    /**
     * 前のページ/次のページがあるかどうか
     * 
     * @return boolean ページがあったらtrue、なければfalse
     */
    protected function _hasPage($how)
    {
        if(!empty($this->params)){
            
            $this->params['nextPage'] = ($this->params['page'] < $this->params['pageCount'])? true : false;
            $this->params['prevPage'] = ($this->params['prev'] > 0)? true : false;
        
            if($this->params["{$how}Page"] == true){
                return true;
            }
        }
        
        return false;
    }
    
    /**
     * ページ番号リンク
     * 
     * @param array $options 
     *      ['tag'] リンクを入れるタグ
     *      ['disp'] 表示する番号の数
     *      ['class'] クラス名
     *      ['separator'] セパレーター 
     */
    public function numbers($options = array())
    {
        $_defaults = array('tag' => 'span', 'disp'=>5, 'class'=>'number', 'separator'=>'');
        $options = array_merge($_defaults, (array)$options);
        
        $start =  ($this->params['page'] - floor($options['disp'] / 2)> 0) ? ($this->params['page'] - floor($options['disp']/2)) : 1;//始点
        $end =  ($start> 1) ? ($this->params['page'] + floor($options['disp']/2)) : $options['disp'];//終点
        $start = ($this->params['pageCount'] <$end)? $start-($end - $this->params['pageCount']) : $start;//始点再計算
       
        for($i=$start; $i <= $end; $i++){
            
            if( ($i <= $this->params['pageCount']) && ($i > 0) ) {
                if($this->params['page'] == $i){
                    print '<' . $options['tag'] . ' class="'.$options['class'].' current">' . $i . '</' . $options['tag'] . '>';
                }else{
                    print '<' . $options['tag'] . ' class="'.$options['class'].'"><a href="'. $this->params['url'] . $this->params['paramKey'] . $i . '">' . $i . '</a></' . $options['tag'] . '>';
                }
                if(($i != $this->params['pageCount']) && ($i != $end)) print $options['separator'];
            }
        }
    }
    
    /**
     * ◎ページ中〇ページ目を表示 
     */
    public function counter($options = array())
    {
        if (is_string($options)) {
            $options = array('format' => $options);
           }
        
        $options = array_merge(
            array(
            'format' => 'pages',
            'separator' => ' of '
        ),
         $options);
         
        $start = 0;
        
        if ($this->params['dataCount'] >= 1) {
            $start = (($this->params['page'] - 1) * $this->params['limit']) + 1;
        }
        
        $end = $start + $this->params['limit'] - 1;
        
        if ($this->params['dataCount'] < $end) {
            $end = $this->params['dataCount'];
        }
        
        switch ($options['format']) {
            case 'pages': //page: 1 of 5
                $out = $this->params['page'] . $options['separator'] . $this->params['pageCount'];
            break;
            
            case 'range'://data : 1－5 of 10
                if (!is_array($options['separator'])) {
                    $options['separator'] = array(' - ', $options['separator']);
                }
                $out = $start . $options['separator'][0] . $end . $options['separator'][1];
                $out .= $this->params['dataCount'];
            break;
            
            default:
                $map = array(
                        '%page%' => $this->params['page'],
                        '%pages%' => $this->params['pageCount'],
                        '%current%' => $this->params['limit'],
                        '%count%' => $this->params['dataCount'],
                        '%start%' => $start,
                        '%end%' => $end
                       );
                       
                $out = str_replace(array_keys($map), array_values($map), $options['format']);
        }
        
        print $out;
    }
    
    /**
     * 現在のページ番号
     */
    public function current(){
        if(isset($this->params['page'])){
            return $this->params['page'];
        }
        
        return 1;
    }
    /**
     * 最初のページへのリンク
     */
    public function first($first = '&laquo; 最初のページへ', $options = array())
    {
        $options = array_merge(
            array(
                'class'=>'first',
                'tag' => 'span'
            ),
            (array)$options);
        
        if ($this->params['page'] <= 1) {
            return false;
        }
        
        extract($options);
        $out = '';
        
        if($this->params['page'] > 1){
            
            $out = '<'.$tag.' class="'.$class.'"><a href="'. $this->params['url'] . $this->params['paramKey'] .'1">'.$first.'</a></'.$tag.'>';
        }
        
        print $out;
    }
    
    /**
     * 最後のページへのリンク
     */
    public function last($last = '最後のページへ &raquo;', $options = array())
    {
        $options = array_merge(
            array(
                'class'=>'last',
                'tag' => 'span'
            ),
            (array)$options);
        
        if ($this->params['pageCount'] <= 1) {
            return false;
        }
        
        extract($options);
        $out = '';
        
        if($this->params['page'] < $this->params['pageCount']){
            
            $out = '<'.$tag.' class="'.$class.'"><a href="'. $this->params['url'] . $this->params['paramKey'] . $this->params['pageCount'].'">'.$last.'</a></'.$tag.'>';
        }
        
        print $out;
    }
}

?>